import java.util.Random;

import org.rlcommunity.rlglue.codec.AgentInterface;
import org.rlcommunity.rlglue.codec.taskspec.TaskSpec;
import org.rlcommunity.rlglue.codec.taskspec.ranges.DoubleRange;
import org.rlcommunity.rlglue.codec.taskspec.ranges.IntRange;
import org.rlcommunity.rlglue.codec.types.Action;
import org.rlcommunity.rlglue.codec.types.Observation;

/**
 * Simple random agent for the recommendation engine task.
 * Agent parses observations but does not learn anything. Instead chooses a random action at every step.
 *
 * Action consists in choosing a page in the shop that will be shown as a banner.
 * Pages are identified as integers between minPage and maxPage.
 *
 * Observations consists of two integer values.
 * First value is the page on which the banner will be displayed: integer between minPage and maxPage.
 * Second value is the current user action:
 * 0: Abort: user left the shop.
 * 1: Purchase: user bought something.
 * 2: Click on banner: user clicked on the last banner.
 * 3: Normal click: user visits a new page.
 *
 * @author Uwe Dick
 *
 */
public class Recom_Agent implements AgentInterface {

	Random randGenerator = new Random();
	/** number of pages in the shop */
	int numPages;
	/** min and max index of pages in the shop */
	int minPage, maxPage;

	/** last action taken by the agent */
	Action lastAction;
	/** last observation seen */
	Observation lastObservation;
	/** last reward seen */
	double lastReward;
	/** reward of the current episode */
	double episodeReward;
	/** step counter for the current episode */
	int step;
	/** episode counter */
	int numEpisode;
	/** page the user clicked on and on which the banner will be displayed */
	int currentPage;
	/** current action of user:
	 * 0: Abort: user left the shop.
	 * 1: Purchase: user bought something.
	 * 2: Click on banner: user clicked on the last banner.
	 * 3: Normal click: user visits a new page.
	 */
	int currentUserAction;

	/** Called at Start of Experiment */
    public void agent_init(String taskSpecification) {
		TaskSpec theTaskSpec=new TaskSpec(taskSpecification);
		System.out.println("Random agent parsed the task spec.");
		System.out.println("Observation have "+theTaskSpec.getNumDiscreteObsDims()+" integer dimensions");
		System.out.println("Actions have "+theTaskSpec.getNumDiscreteActionDims()+" integer dimensions");
		IntRange theObsRange=theTaskSpec.getDiscreteObservationRange(0);
		System.out.println("Observation[0] range is: "+theObsRange.getMin()+" to "+theObsRange.getMax());
		theObsRange=theTaskSpec.getDiscreteObservationRange(1);
		System.out.println("Observation[1] range is: "+theObsRange.getMin()+" to "+theObsRange.getMax());

		IntRange theActRange=theTaskSpec.getDiscreteActionRange(0);
		minPage = theActRange.getMin();
		maxPage = theActRange.getMax();
		numPages = maxPage - minPage + 1;
		System.out.println("Action range is: "+theActRange.getMin()+" to "+theActRange.getMax());
		DoubleRange theRewardRange=theTaskSpec.getRewardRange();
		System.out.println("Reward range is: "+theRewardRange.getMin()+" to "+theRewardRange.getMax());

		numEpisode = 0;
    }

	/** Called at start of each episode */
	public Action agent_start(Observation observation) {
		currentPage = observation.getInt(0);
		currentUserAction = observation.getInt(1);

	    /** Choose random Action */
        int theIntAction = randGenerator.nextInt(numPages) + minPage;
        /**
         * Create a structure to hold 1 integer action
         * and set the value
         */
        Action returnAction = new Action(1, 0, 0);
        returnAction.intArray[0] = theIntAction;

        lastAction = returnAction.duplicate();
        lastObservation = observation.duplicate();
        lastReward = 0;
        episodeReward = 0;
        step = 0;

        numEpisode++;

        return returnAction;
	}

	/** Presents observation and reward to agent. Agent needs to take an action. */
	public Action agent_step(double reward, Observation observation) {
		currentPage = observation.getInt(0);
		currentUserAction = observation.getInt(1);
		switch(currentUserAction) {
		case 0:
			// User left the Shop. Abort
			break;
		case 1:
			// User bought something on 'currentPage'.
			break;
		case 2:
			// User clicked on the banner. (which should be 'currentPage')
			break;
		case 3:
			// User clicked on 'currentPage' not via banner.
		}

		/** Choose random Action */
        int theIntAction = randGenerator.nextInt(numPages) + minPage;
        /**
         * Create a structure to hold 1 integer action
         * and set the value
         */
        Action returnAction = new Action(1, 0, 0);
        returnAction.intArray[0] = theIntAction;

        lastAction = returnAction.duplicate();
        lastObservation = observation.duplicate();
        lastReward = reward;
        episodeReward += reward;
        step ++;

        return returnAction;
	}

	/** Called at end of each episode */
	public void agent_end(double reward) {
		episodeReward += reward;
		step ++;
//		System.out.println("Agent:  Episode "+numEpisode+":  Reward: "+episodeReward+"  Steps: "+step);
	}

	/** Called at end of experiment */
	public void agent_cleanup() {
		lastAction=null;
		lastObservation=null;
		lastReward = 0;
	}

	public String agent_message(String arg0) {
		if(arg0.equals("stats")) {
			return "Agent:  Episode "+numEpisode+":  Reward: "+episodeReward+"  Steps: "+step;
		}
		return null;
	}

}
